#include <random>
#include <algorithm>
#include <iostream>
#include <vector>
#include <omp.h>
#include <cmath>

using namespace std;

int fnNumSamplesInCircle(int randomSeed, int numSamples) {
    mt19937 randomEngine(randomSeed);
    uniform_real_distribution<float> dist(-1.0, 1.0);

    int numSamplesInCircle = 0;
    for (int sampleIndex = 0; sampleIndex < numSamples; sampleIndex++) {
        float x = dist(randomEngine);
        float y = dist(randomEngine);
        if ((x*x + y*y) <= 1.0) {
            numSamplesInCircle++;
        }
    }

    return numSamplesInCircle;
}

int main() {
    int totalNumSamples = 1000000000;

    int numIterations = 100;
    int numSamplesPerIteration = totalNumSamples / numIterations;
    if ((totalNumSamples % numIterations) != 0) {
        cout << "Total number of samples must be multiple of number of iterations!" << endl;
        exit(1);
    }

    int numSamplesInCircle = 0;
    #pragma omp parallel for reduction(+:numSamplesInCircle)
    for (int i = 0; i < numIterations; i++) {
        numSamplesInCircle += fnNumSamplesInCircle(i, numSamplesPerIteration);
    }

    cout << (4.0 * numSamplesInCircle) / (1.0 * totalNumSamples) << endl;
    return 0;
}
